<?php
/*
Copyright (c) 2014, Multon OÜ
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of the FreeBSD Project.
*/
class Multon_Checkstension_Helper_Data extends Mage_Core_Helper_Abstract {

    public function getTables()
    {

        $db = Mage::getConfig()->getResourceConnectionConfig('default_setup')->dbname;

        $resource = Mage::getSingleton('core/resource');
        $readConnection = $resource->getConnection('core_read');
        $query = '
            SELECT
                table_name AS "Table",
                data_length + index_length AS "Size"
            FROM
                information_schema.TABLES
            WHERE
                table_schema = "' . $db .'"
            ORDER BY
                Size DESC
        ';
        $results = $readConnection->fetchAll($query);

        return $results;
        
    }

    public function getPHPInfo()
    {
        ob_start();
        phpinfo();
        $info_arr = array();
        $info_lines = explode("\n", strip_tags(ob_get_clean(), "<tr><td><h2>"));
        $cat = "General";
        foreach ($info_lines as $line) {
            // new cat?
            preg_match("~<h2>(.*)</h2>~", $line, $title) ? $cat = $title[1] : null;
            if (preg_match("~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~", $line, $val)) {
                $info_arr[$cat][$val[1]] = $val[2];
            } elseif (preg_match("~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~", $line, $val)) {
                $info_arr[$cat][$val[1]] = array("local" => $val[2], "master" => $val[3]);
            }
        }
        return $info_arr;
    }

    public function shortenBytes($bytes)
    {
        $symbols = array('B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB');
        $exp = $bytes ? floor(log($bytes) / log(1024)) : 0;

        return sprintf('%.2f ' . $symbols[$exp], ($bytes / pow(1024, floor($exp))));
    }

    public function getExtensionVersion()
    {
        return (string) Mage::getConfig()->getNode()->modules->Multon_Checkstension->version;
    }

    public function getMemory()
    {
        return ini_get('memory_limit');
    }

    public function isEnough()
    {
        $limit = $this->getMemory();
        if (preg_match('/^(\d+)(.)$/', $limit, $matches)) {
            if ($matches[2] == 'M') {
                $limit = $matches[1] * 1024 * 1024; // nnnM -> nnn MB
            } else if ($matches[2] == 'K') {
                $limit = $matches[1] * 1024; // nnnK -> nnn KB
            }
        }

        if ($limit < 128 * 1024 * 1024) {
            return 0; // Less than 128MB (not enough memory)
        } else if ($limit < 512 * 1024 * 1024) {
            return 1; // Less than 512MB (works, but barely)
        }

        return 2; // 512MB or more (recommended value)
    }

    public function getExtensions()
    {
        // Path data
        $base_path = Mage::getBaseDir('base');
        $modules_path = '/app/etc/modules/';
        $code_path = '/app/code/';
        // Return list
        $extensions = array();
        // Open modules folder
        $dir = opendir($base_path . $modules_path);
        if ($dir) {
            while (($file = readdir($dir)) !== false) {
                // Ignore folders
                if (!is_dir($base_path . $modules_path . $file)) {
                    // Load only files with .xml extension
                    if (substr(strtolower($file), -4) == '.xml') {
                        // Load module file
                        $xml = simplexml_load_file(
                                $base_path . $modules_path . $file);
                        $modules = $xml->modules->children();
                        // Parse through all modules found
                        foreach ($modules as $key => $module) {
                            $active = (string) $module->active;
                            $codePool = (string) $module->codePool;
                            // Ignore core modules
                            if ($codePool != 'core') {
                                $extension = array();
                                $extension['name'] = $key;
                                $extension['active'] = $active;
                                $extension['codePool'] = $codePool;
                                $ext_path = str_replace('_', '/', $key);
                                $config_path = $base_path . $code_path .
                                        $codePool . '/' . $ext_path .
                                        '/etc/config.xml';
                                // Load extension config
                                $config = simplexml_load_file($config_path);
                                if ($config) {
                                    $extension['version'] = $config->modules->$key->version;
                                } else {
                                    $extension['msg'] = $this->__('Not found');
                                }
                                $extensions[] = $extension;
                            }
                        }
                    }
                }
            }
            closedir($dir);
        }
        return $extensions;
    }

    public function formatInterval($interval)
    {
        $output = '';

        if ($interval->y)
            $output .= $interval->y . ' ' . $this->__('years') . ', ';
        if ($output || $interval->m)
            $output .= $interval->m . ' ' . $this->__('months') . ', ';
        if ($output || $interval->d)
            $output .= $interval->d . ' ' . $this->__('days') . ', ';
        if ($output || $interval->h)
            $output .= $interval->h . ' ' . $this->__('hours') . ', ';
        if ($output || $interval->i)
            $output .= $interval->i . ' ' . $this->__('minutes') . ', ';
        if ($output || $interval->s)
            $output .= $interval->s . ' ' . $this->__('seconds');

        return $output;
    }

    public function colorDiff($date)
    {
        $warning = $error = $date;
        $warning->add(new DateInterval('PT1H'));
        $error->add(new DateInterval('PT6H'));
        $now = new DateTime();
        if ($error < $now) {
            return 0;
        } else if ($warning < $now) {
            return 1;
        } else {
            return 2;
        }
    }

}
